@@ -4,9 +4,11 @@
 #if PY_MAJOR_VERSION >= 3
 #define PySass_IF_PY3(three, two) (three)
 #define PySass_Object_Bytes(o) PyUnicode_AsUTF8String(PyObject_Str(o))
+#define COLLECTIONS_ABC_MOD "collections.abc"
 #else
 #define PySass_IF_PY3(three, two) (two)
 #define PySass_Object_Bytes(o) PyObject_Str(o)
+#define COLLECTIONS_ABC_MOD "collections"
 #endif
 
 #ifdef __cplusplus
@@ -322,7 +324,7 @@ static union Sass_Value* _to_sass_value(PyObject* value) {
     PyObject* sass_list_t = PyObject_GetAttrString(types_mod, "SassList");
     PyObject* sass_warning_t = PyObject_GetAttrString(types_mod, "SassWarning");
     PyObject* sass_error_t = PyObject_GetAttrString(types_mod, "SassError");
-    PyObject* collections_mod = PyImport_ImportModule("collections");
+    PyObject* collections_mod = PyImport_ImportModule(COLLECTIONS_ABC_MOD);
     PyObject* mapping_t = PyObject_GetAttrString(collections_mod, "Mapping");
 
     if (value == Py_None) {
@@ -505,37 +507,29 @@ static void _add_custom_importers(
     sass_option_set_c_importers(options, importer_list);
 }
 
-static void _add_custom_import_extensions(
-        struct Sass_Options* options, PyObject* custom_import_extensions
-) {
-    Py_ssize_t i;
-
-    for (i = 0; i < PyList_GET_SIZE(custom_import_extensions); i += 1) {
-        PyObject* ext = PyList_GET_ITEM(custom_import_extensions, i);
-        sass_option_push_import_extension(options, PyBytes_AS_STRING(ext));
-    }
-}
-
 static PyObject *
 PySass_compile_string(PyObject *self, PyObject *args) {
     struct Sass_Context *ctx;
     struct Sass_Data_Context *context;
     struct Sass_Options *options;
-    char *string, *include_paths;
+    char *string, *include_paths, *source_map_file;
     const char *error_message, *output_string;
     Sass_Output_Style output_style;
-    int source_comments, error_status, precision, indented;
+    int source_comments, error_status, precision, indented,
+        source_map_embed, source_map_contents, source_map_file_urls,
+        omit_source_map_url;
     PyObject *custom_functions;
     PyObject *custom_importers;
-    PyObject *custom_import_extensions;
+    PyObject *source_map_root;
     PyObject *result;
 
     if (!PyArg_ParseTuple(args,
-                          PySass_IF_PY3("yiiyiOiOO", "siisiOiOO"),
+                          PySass_IF_PY3("yiiyiOiOiiiO", "siisiOiOiiiO"),
                           &string, &output_style, &source_comments,
                           &include_paths, &precision,
                           &custom_functions, &indented, &custom_importers,
-                          &custom_import_extensions)) {
+                          &source_map_contents, &source_map_embed,
+                          &omit_source_map_url, &source_map_root)) {
         return NULL;
     }
 
@@ -546,9 +540,18 @@ PySass_compile_string(PyObject *self, PyObject *args) {
     sass_option_set_include_path(options, include_paths);
     sass_option_set_precision(options, precision);
     sass_option_set_is_indented_syntax_src(options, indented);
+    sass_option_set_source_map_contents(options, source_map_contents);
+    sass_option_set_source_map_embed(options, source_map_embed);
+    sass_option_set_omit_source_map_url(options, omit_source_map_url);
+
+    if (PyBytes_Check(source_map_root) && PyBytes_GET_SIZE(source_map_root)) {
+        sass_option_set_source_map_root(
+            options, PyBytes_AS_STRING(source_map_root)
+        );
+    }
+
     _add_custom_functions(options, custom_functions);
     _add_custom_importers(options, custom_importers);
-    _add_custom_import_extensions(options, custom_import_extensions);
     sass_compile_data_context(context);
 
     ctx = sass_data_context_get_context(context);
@@ -572,17 +575,19 @@ PySass_compile_filename(PyObject *self, PyObject *args) {
     char *filename, *include_paths;
     const char *error_message, *output_string, *source_map_string;
     Sass_Output_Style output_style;
-    int source_comments, error_status, precision;
+    int source_comments, error_status, precision, source_map_embed,
+        source_map_contents, source_map_file_urls, omit_source_map_url;
     PyObject *source_map_filename, *custom_functions, *custom_importers,
-             *result, *output_filename_hint, *custom_import_extensions;
+             *result, *output_filename_hint, *source_map_root;
 
     if (!PyArg_ParseTuple(args,
-                          PySass_IF_PY3("yiiyiOOOOO", "siisiOOOOO"),
+                          PySass_IF_PY3("yiiyiOOOOiiiO", "siisiOOOOiiiO"),
                           &filename, &output_style, &source_comments,
                           &include_paths, &precision,
                           &source_map_filename, &custom_functions,
                           &custom_importers, &output_filename_hint,
-                          &custom_import_extensions)) {
+                          &source_map_contents, &source_map_embed,
+                          &omit_source_map_url, &source_map_root)) {
         return NULL;
     }
 
@@ -603,13 +608,22 @@ PySass_compile_filename(PyObject *self, PyObject *args) {
             );
         }
     }
+
+    if (PyBytes_Check(source_map_root) && PyBytes_GET_SIZE(source_map_root)) {
+        sass_option_set_source_map_root(
+            options, PyBytes_AS_STRING(source_map_root)
+        );
+    }
+
     sass_option_set_output_style(options, output_style);
     sass_option_set_source_comments(options, source_comments);
     sass_option_set_include_path(options, include_paths);
     sass_option_set_precision(options, precision);
+    sass_option_set_source_map_contents(options, source_map_contents);
+    sass_option_set_source_map_embed(options, source_map_embed);
+    sass_option_set_omit_source_map_url(options, omit_source_map_url);
     _add_custom_functions(options, custom_functions);
     _add_custom_importers(options, custom_importers);
-    _add_custom_import_extensions(options, custom_import_extensions);
     sass_compile_file_context(context);
 
     ctx = sass_file_context_get_context(context);
